"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const maps_1 = require("./utils/collections/maps");
/**
 * Enforcing view on an entity loader. All loads through this loader will throw
 * if the loads are not successful.
 */
class EnforcingEntityLoader {
    constructor(entityLoader) {
        this.entityLoader = entityLoader;
    }
    /**
     * Enforcing version of entity loader method by the same name.
     * @throws {@link EntityNotAuthorizedError} when viewer is not authorized to view one or more of the returned entities
     */
    async loadManyByFieldEqualingManyAsync(fieldName, fieldValues) {
        const fieldValuesToResults = await this.entityLoader.loadManyByFieldEqualingManyAsync(fieldName, fieldValues);
        return maps_1.mapMap(fieldValuesToResults, (results) => results.map((result) => result.enforceValue()));
    }
    /**
     * Enforcing version of entity loader method by the same name.
     * @throws {@link EntityNotAuthorizedError} when viewer is not authorized to view one or more of the returned entities
     */
    async loadManyByFieldEqualingAsync(fieldName, fieldValue) {
        const entityResults = await this.entityLoader.loadManyByFieldEqualingAsync(fieldName, fieldValue);
        return entityResults.map((result) => result.enforceValue());
    }
    /**
     * Enforcing version of entity loader method by the same name.
     * @throws {@link EntityNotAuthorizedError} when viewer is not authorized to view the returned entity
     * @throws when multiple entities are found matching the condition
     */
    async loadByFieldEqualingAsync(uniqueFieldName, fieldValue) {
        const entityResult = await this.entityLoader.loadByFieldEqualingAsync(uniqueFieldName, fieldValue);
        return entityResult ? entityResult.enforceValue() : null;
    }
    /**
     * Enforcing version of entity loader method by the same name.
     * @throws {@link EntityNotAuthorizedError} when viewer is not authorized to view the returned entity
     */
    async loadByIDAsync(id) {
        const entityResult = await this.entityLoader.loadByIDAsync(id);
        return entityResult.enforceValue();
    }
    /**
     * Enforcing version of entity loader method by the same name.
     * @throws {@link EntityNotAuthorizedError} when viewer is not authorized to view the returned entity
     * @throws when multiple entities are found matching the condition
     */
    async loadByIDNullableAsync(id) {
        const entityResult = await this.entityLoader.loadByIDNullableAsync(id);
        return entityResult ? entityResult.enforceValue() : null;
    }
    /**
     * Enforcing version of entity loader method by the same name.
     * @throws {@link EntityNotAuthorizedError} when viewer is not authorized to view one or more of the returned entities
     */
    async loadManyByIDsAsync(ids) {
        const entityResults = await this.entityLoader.loadManyByIDsAsync(ids);
        return maps_1.mapMap(entityResults, (result) => result.enforceValue());
    }
    /**
     * Enforcing version of entity loader method by the same name.
     * @throws {@link EntityNotAuthorizedError} when viewer is not authorized to view one or more of the returned entities
     */
    async loadManyByFieldEqualityConjunctionAsync(fieldEqualityOperands, querySelectionModifiers = {}) {
        const entityResults = await this.entityLoader.loadManyByFieldEqualityConjunctionAsync(fieldEqualityOperands, querySelectionModifiers);
        return entityResults.map((result) => result.enforceValue());
    }
    /**
     * Enforcing version of entity loader method by the same name.
     * @throws {@link EntityNotAuthorizedError} when viewer is not authorized to view one or more of the returned entities
     */
    async loadManyByRawWhereClauseAsync(rawWhereClause, bindings, querySelectionModifiers = {}) {
        const entityResults = await this.entityLoader.loadManyByRawWhereClauseAsync(rawWhereClause, bindings, querySelectionModifiers);
        return entityResults.map((result) => result.enforceValue());
    }
}
exports.default = EnforcingEntityLoader;
//# sourceMappingURL=EnforcingEntityLoader.js.map