"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const EntityLoaderFactory_1 = __importDefault(require("./EntityLoaderFactory"));
const EntityMutatorFactory_1 = __importDefault(require("./EntityMutatorFactory"));
/**
 * Composition root responsible for orchestrating setup of Entity mutators and loaders.
 */
class EntityCompanion {
    constructor(entityClass, tableDataCoordinator, PrivacyPolicyClass, mutationValidators, mutationTriggers, metricsAdapter) {
        this.tableDataCoordinator = tableDataCoordinator;
        const privacyPolicy = new PrivacyPolicyClass();
        this.entityLoaderFactory = new EntityLoaderFactory_1.default(tableDataCoordinator.entityConfiguration, entityClass, privacyPolicy, tableDataCoordinator.dataManager);
        this.entityMutatorFactory = new EntityMutatorFactory_1.default(tableDataCoordinator.entityConfiguration, entityClass, privacyPolicy, mutationValidators, mutationTriggers, this.entityLoaderFactory, tableDataCoordinator.databaseAdapter, metricsAdapter);
    }
    getLoaderFactory() {
        return this.entityLoaderFactory;
    }
    getMutatorFactory() {
        return this.entityMutatorFactory;
    }
    /**
     * Get the query context provider for this entity.
     */
    getQueryContextProvider() {
        return this.tableDataCoordinator.getQueryContextProvider();
    }
}
exports.default = EntityCompanion;
//# sourceMappingURL=EntityCompanion.js.map