"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const maps_1 = require("./utils/collections/maps");
/**
 * The data storage configuration for a type of Entity. Contains information relating to IDs,
 * cachable fields, field mappings, and types of cache and database adapter.
 */
class EntityConfiguration {
    constructor({ idField, tableName, schema, getInboundEdges = () => [], cacheKeyVersion = 0, databaseAdapterFlavor, cacheAdapterFlavor, }) {
        this.idField = idField;
        this.tableName = tableName;
        this.cacheKeyVersion = cacheKeyVersion;
        this.databaseAdapterFlavor = databaseAdapterFlavor;
        this.cacheAdapterFlavor = cacheAdapterFlavor;
        this.getInboundEdges = getInboundEdges;
        // external schema is a Record to typecheck that all fields have FieldDefinitions,
        // but internally the most useful representation is a map for lookups
        // TODO(wschurman): validate schema
        this.schema = new Map(Object.entries(schema));
        this.cacheableKeys = EntityConfiguration.computeCacheableKeys(this.schema);
        this.entityToDBFieldsKeyMapping = EntityConfiguration.computeEntityToDBFieldsKeyMapping(this.schema);
        this.dbToEntityFieldsKeyMapping = maps_1.invertMap(this.entityToDBFieldsKeyMapping);
    }
    static computeCacheableKeys(schema) {
        return maps_1.reduceMap(schema, (acc, v, k) => {
            if (v.cache) {
                acc.add(k);
            }
            return acc;
        }, new Set());
    }
    static computeEntityToDBFieldsKeyMapping(schema) {
        return maps_1.mapMap(schema, (v) => v.columnName);
    }
}
exports.default = EntityConfiguration;
//# sourceMappingURL=EntityConfiguration.js.map