import { IEntityClass } from './Entity';
import EntityConfiguration from './EntityConfiguration';
import EntityLoader from './EntityLoader';
import EntityPrivacyPolicy from './EntityPrivacyPolicy';
import { EntityQueryContext } from './EntityQueryContext';
import ReadonlyEntity from './ReadonlyEntity';
import ViewerContext from './ViewerContext';
import EntityDataManager from './internal/EntityDataManager';
/**
 * The primary entry point for loading entities.
 */
export default class EntityLoaderFactory<TFields, TID extends NonNullable<TFields[TSelectedFields]>, TViewerContext extends ViewerContext, TEntity extends ReadonlyEntity<TFields, TID, TViewerContext, TSelectedFields>, TPrivacyPolicy extends EntityPrivacyPolicy<TFields, TID, TViewerContext, TEntity, TSelectedFields>, TSelectedFields extends keyof TFields> {
    private readonly entityConfiguration;
    private readonly entityClass;
    private readonly privacyPolicyClass;
    private readonly dataManager;
    constructor(entityConfiguration: EntityConfiguration<TFields>, entityClass: IEntityClass<TFields, TID, TViewerContext, TEntity, TPrivacyPolicy, TSelectedFields>, privacyPolicyClass: TPrivacyPolicy, dataManager: EntityDataManager<TFields>);
    /**
     * Vend loader for loading an entity in a given query context.
     * @param viewerContext - viewer context of loading user
     * @param queryContext - query context in which to perform the load
     */
    forLoad(viewerContext: TViewerContext, queryContext: EntityQueryContext): EntityLoader<TFields, TID, TViewerContext, TEntity, TPrivacyPolicy, TSelectedFields>;
}
