import EntityQueryContextProvider from './EntityQueryContextProvider';
export declare type PostCommitCallback = (...args: any) => Promise<any>;
/**
 * Entity framework representation of transactional and non-transactional database
 * query execution units.
 *
 * The behavior of {@link EntityMutator} and {@link EntityLoader}
 * differs when in a transactional context.
 */
export declare abstract class EntityQueryContext {
    private readonly queryInterface;
    constructor(queryInterface: any);
    abstract isInTransaction(): boolean;
    getQueryInterface(): any;
    abstract runInTransactionIfNotInTransactionAsync<T>(transactionScope: (queryContext: EntityTransactionalQueryContext) => Promise<T>): Promise<T>;
}
export declare class EntityNonTransactionalQueryContext extends EntityQueryContext {
    private readonly entityQueryContextProvider;
    constructor(queryInterface: any, entityQueryContextProvider: EntityQueryContextProvider);
    isInTransaction(): boolean;
    runInTransactionIfNotInTransactionAsync<T>(transactionScope: (queryContext: EntityTransactionalQueryContext) => Promise<T>): Promise<T>;
}
export declare class EntityTransactionalQueryContext extends EntityQueryContext {
    private readonly postCommitCallbacks;
    appendPostCommitCallback(callback: PostCommitCallback): void;
    runPostCommitCallbacksAsync(): Promise<void>;
    isInTransaction(): boolean;
    runInTransactionIfNotInTransactionAsync<T>(transactionScope: (queryContext: EntityTransactionalQueryContext) => Promise<T>): Promise<T>;
}
