"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EntityTransactionalQueryContext = exports.EntityNonTransactionalQueryContext = exports.EntityQueryContext = void 0;
/**
 * Entity framework representation of transactional and non-transactional database
 * query execution units.
 *
 * The behavior of {@link EntityMutator} and {@link EntityLoader}
 * differs when in a transactional context.
 */
class EntityQueryContext {
    constructor(queryInterface) {
        this.queryInterface = queryInterface;
    }
    getQueryInterface() {
        return this.queryInterface;
    }
}
exports.EntityQueryContext = EntityQueryContext;
class EntityNonTransactionalQueryContext extends EntityQueryContext {
    constructor(queryInterface, entityQueryContextProvider) {
        super(queryInterface);
        this.entityQueryContextProvider = entityQueryContextProvider;
    }
    isInTransaction() {
        return false;
    }
    async runInTransactionIfNotInTransactionAsync(transactionScope) {
        return await this.entityQueryContextProvider.runInTransactionAsync(transactionScope);
    }
}
exports.EntityNonTransactionalQueryContext = EntityNonTransactionalQueryContext;
class EntityTransactionalQueryContext extends EntityQueryContext {
    constructor() {
        super(...arguments);
        this.postCommitCallbacks = [];
    }
    appendPostCommitCallback(callback) {
        this.postCommitCallbacks.push(callback);
    }
    async runPostCommitCallbacksAsync() {
        const callbacks = [...this.postCommitCallbacks];
        this.postCommitCallbacks.length = 0;
        await Promise.all(callbacks.map((callback) => callback()));
    }
    isInTransaction() {
        return true;
    }
    async runInTransactionIfNotInTransactionAsync(transactionScope) {
        return await transactionScope(this);
    }
}
exports.EntityTransactionalQueryContext = EntityTransactionalQueryContext;
//# sourceMappingURL=EntityQueryContext.js.map