"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const EntityQueryContext_1 = require("./EntityQueryContext");
/**
 * A query context provider vends transactional and non-transactional query contexts.
 */
class EntityQueryContextProvider {
    /**
     * Vend a regular (non-transactional) entity query context.
     */
    getQueryContext() {
        return new EntityQueryContext_1.EntityNonTransactionalQueryContext(this.getQueryInterface(), this);
    }
    /**
     * Start a transaction and execute the provided transaction-scoped closure within the transaction.
     * @param transactionScope - async callback to execute within the transaction
     */
    async runInTransactionAsync(transactionScope) {
        const [returnedValue, queryContext] = await this.createTransactionRunner()(async (queryInterface) => {
            const queryContext = new EntityQueryContext_1.EntityTransactionalQueryContext(queryInterface);
            const result = await transactionScope(queryContext);
            return [result, queryContext];
        });
        await queryContext.runPostCommitCallbacksAsync();
        return returnedValue;
    }
}
exports.default = EntityQueryContextProvider;
//# sourceMappingURL=EntityQueryContextProvider.js.map