"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const results_1 = require("@expo/results");
const ts_mockito_1 = require("ts-mockito");
const EnforcingEntityLoader_1 = __importDefault(require("../EnforcingEntityLoader"));
const EntityLoader_1 = __importDefault(require("../EntityLoader"));
describe(EnforcingEntityLoader_1.default, () => {
    describe('loadManyByFieldEqualingManyAsync', () => {
        it('throws when result is unsuccessful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const rejection = new Error();
            ts_mockito_1.when(entityLoaderMock.loadManyByFieldEqualingManyAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve(new Map(Object.entries({
                hello: [results_1.result(rejection)],
            })));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByFieldEqualingManyAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).rejects.toThrow(rejection);
        });
        it('returns value when result is successful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = {};
            ts_mockito_1.when(entityLoaderMock.loadManyByFieldEqualingManyAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve(new Map(Object.entries({
                hello: [results_1.result(resolved)],
            })));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByFieldEqualingManyAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).resolves.toEqual(new Map(Object.entries({
                hello: [resolved],
            })));
        });
    });
    describe('loadManyByFieldEqualingAsync', () => {
        it('throws when result is unsuccessful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const rejection = new Error();
            ts_mockito_1.when(entityLoaderMock.loadManyByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve([
                results_1.result(rejection),
            ]);
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).rejects.toThrow(rejection);
        });
        it('returns value when result is successful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = {};
            ts_mockito_1.when(entityLoaderMock.loadManyByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve([
                results_1.result(resolved),
            ]);
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).resolves.toEqual([resolved]);
        });
    });
    describe('loadByFieldEqualingAsync', () => {
        it('throws when result is unsuccessful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const rejection = new Error();
            ts_mockito_1.when(entityLoaderMock.loadByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve(results_1.result(rejection));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).rejects.toThrow(rejection);
        });
        it('returns value when result is successful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = {};
            ts_mockito_1.when(entityLoaderMock.loadByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve(results_1.result(resolved));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).resolves.toEqual(resolved);
        });
        it('returns null when result is successful and no entity is found', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = null;
            ts_mockito_1.when(entityLoaderMock.loadByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve(results_1.result(resolved));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).resolves.toEqual(resolved);
        });
        it('throws when multiple matching entities are found', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const multipleEntitiesError = new Error();
            ts_mockito_1.when(entityLoaderMock.loadByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenReject(multipleEntitiesError);
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).rejects.toEqual(multipleEntitiesError);
        });
    });
    describe('loadByIDAsync', () => {
        it('throws when result is unsuccessful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const rejection = new Error();
            ts_mockito_1.when(entityLoaderMock.loadByIDAsync(ts_mockito_1.anything())).thenResolve(results_1.result(rejection));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByIDAsync(ts_mockito_1.anything())).rejects.toThrow(rejection);
        });
        it('returns value when result is successful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = {};
            ts_mockito_1.when(entityLoaderMock.loadByIDAsync(ts_mockito_1.anything())).thenResolve(results_1.result(resolved));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByIDAsync(ts_mockito_1.anything())).resolves.toEqual(resolved);
        });
    });
    describe('loadByIDNullableAsync', () => {
        it('throws when result is unsuccessful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const rejection = new Error();
            ts_mockito_1.when(entityLoaderMock.loadByIDNullableAsync(ts_mockito_1.anything())).thenResolve(results_1.result(rejection));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByIDNullableAsync(ts_mockito_1.anything())).rejects.toThrow(rejection);
        });
        it('returns value when result is successful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = {};
            ts_mockito_1.when(entityLoaderMock.loadByIDNullableAsync(ts_mockito_1.anything())).thenResolve(results_1.result(resolved));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByIDNullableAsync(ts_mockito_1.anything())).resolves.toEqual(resolved);
        });
        it('returns null when non-existent object', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = null;
            ts_mockito_1.when(entityLoaderMock.loadByIDNullableAsync(ts_mockito_1.anything())).thenResolve(results_1.result(resolved));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadByIDNullableAsync(ts_mockito_1.anything())).resolves.toEqual(resolved);
        });
    });
    describe('loadManyByIDsAsync', () => {
        it('throws when result is unsuccessful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const rejection = new Error();
            ts_mockito_1.when(entityLoaderMock.loadManyByIDsAsync(ts_mockito_1.anything())).thenResolve(new Map(Object.entries({
                hello: results_1.result(rejection),
            })));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByIDsAsync(ts_mockito_1.anything())).rejects.toThrow(rejection);
        });
        it('returns value when result is successful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = {};
            ts_mockito_1.when(entityLoaderMock.loadManyByIDsAsync(ts_mockito_1.anything())).thenResolve(new Map(Object.entries({
                hello: results_1.result(resolved),
            })));
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByIDsAsync(ts_mockito_1.anything())).resolves.toEqual(new Map(Object.entries({
                hello: resolved,
            })));
        });
    });
    describe('loadManyByFieldEqualityConjunction', () => {
        it('throws when result is unsuccessful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const rejection = new Error();
            ts_mockito_1.when(entityLoaderMock.loadManyByFieldEqualityConjunctionAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve([results_1.result(rejection)]);
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByFieldEqualityConjunctionAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).rejects.toThrow(rejection);
        });
        it('returns value when result is successful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = {};
            ts_mockito_1.when(entityLoaderMock.loadManyByFieldEqualityConjunctionAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve([results_1.result(resolved)]);
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByFieldEqualityConjunctionAsync(ts_mockito_1.anything(), ts_mockito_1.anything())).resolves.toEqual([resolved]);
        });
    });
    describe('loadManyByRawWhereClause', () => {
        it('throws when result is unsuccessful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const rejection = new Error();
            ts_mockito_1.when(entityLoaderMock.loadManyByRawWhereClauseAsync(ts_mockito_1.anything(), ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve([results_1.result(rejection)]);
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByRawWhereClauseAsync(ts_mockito_1.anything(), ts_mockito_1.anything(), ts_mockito_1.anything())).rejects.toThrow(rejection);
        });
        it('returns value when result is successful', async () => {
            const entityLoaderMock = ts_mockito_1.mock(EntityLoader_1.default);
            const resolved = {};
            ts_mockito_1.when(entityLoaderMock.loadManyByRawWhereClauseAsync(ts_mockito_1.anything(), ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve([results_1.result(resolved)]);
            const entityLoader = ts_mockito_1.instance(entityLoaderMock);
            const enforcingEntityLoader = new EnforcingEntityLoader_1.default(entityLoader);
            await expect(enforcingEntityLoader.loadManyByRawWhereClauseAsync(ts_mockito_1.anything(), ts_mockito_1.anything(), ts_mockito_1.anything())).resolves.toEqual([resolved]);
        });
    });
    it('has the same method names as EntityLoader', () => {
        const enforcingLoaderProperties = Object.getOwnPropertyNames(EnforcingEntityLoader_1.default.prototype);
        const loaderProperties = Object.getOwnPropertyNames(EntityLoader_1.default.prototype);
        // ensure known differences still exist for sanity check
        const knownLoaderOnlyDifferences = [
            'enforcing',
            'invalidateFieldsAsync',
            'invalidateEntityAsync',
            'tryConstructEntities',
            'validateFieldValues',
            'constructAndAuthorizeEntitiesAsync',
        ];
        expect(loaderProperties).toEqual(expect.arrayContaining(knownLoaderOnlyDifferences));
        const loaderPropertiesWithoutKnownDifferences = loaderProperties.filter((p) => !knownLoaderOnlyDifferences.includes(p));
        expect(enforcingLoaderProperties).toEqual(loaderPropertiesWithoutKnownDifferences);
    });
});
//# sourceMappingURL=EnforcingEntityLoader-test.js.map