"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Entity_1 = __importDefault(require("../Entity"));
const EntityCompanionProvider_1 = require("../EntityCompanionProvider");
const EntityConfiguration_1 = __importDefault(require("../EntityConfiguration"));
const EntityFields_1 = require("../EntityFields");
const EntityMutator_1 = require("../EntityMutator");
const EntityPrivacyPolicy_1 = __importDefault(require("../EntityPrivacyPolicy"));
const ViewerContext_1 = __importDefault(require("../ViewerContext"));
const AlwaysAllowPrivacyPolicyRule_1 = __importDefault(require("../rules/AlwaysAllowPrivacyPolicyRule"));
const AlwaysDenyPrivacyPolicyRule_1 = __importDefault(require("../rules/AlwaysDenyPrivacyPolicyRule"));
const SimpleTestEntity_1 = __importDefault(require("../testfixtures/SimpleTestEntity"));
const createUnitTestEntityCompanionProvider_1 = require("../utils/testing/createUnitTestEntityCompanionProvider");
describe(Entity_1.default, () => {
    describe('creator', () => {
        it('creates a new CreateMutator', () => {
            const companionProvider = createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider();
            const viewerContext = new ViewerContext_1.default(companionProvider);
            expect(SimpleTestEntity_1.default.creator(viewerContext)).toBeInstanceOf(EntityMutator_1.CreateMutator);
        });
    });
    describe('updater', () => {
        it('creates a new UpdateMutator', () => {
            const companionProvider = createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider();
            const viewerContext = new ViewerContext_1.default(companionProvider);
            const data = {
                id: 'what',
            };
            const testEntity = new SimpleTestEntity_1.default(viewerContext, data);
            expect(SimpleTestEntity_1.default.updater(testEntity)).toBeInstanceOf(EntityMutator_1.UpdateMutator);
        });
    });
    describe('canViewerUpdateAsync', () => {
        it('appropriately executes update privacy policy', async () => {
            const companionProvider = createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider();
            const viewerContext = new ViewerContext_1.default(companionProvider);
            const data = {
                id: 'what',
            };
            const testEntity = new SimpleTestDenyDeleteEntity(viewerContext, data);
            const canViewerUpdate = await SimpleTestDenyDeleteEntity.canViewerUpdateAsync(testEntity);
            expect(canViewerUpdate).toBe(true);
        });
        it('denies when policy denies', async () => {
            const companionProvider = createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider();
            const viewerContext = new ViewerContext_1.default(companionProvider);
            const data = {
                id: 'what',
            };
            const testEntity = new SimpleTestDenyUpdateEntity(viewerContext, data);
            const canViewerUpdate = await SimpleTestDenyUpdateEntity.canViewerUpdateAsync(testEntity);
            expect(canViewerUpdate).toBe(false);
        });
    });
    describe('canViewerDeleteAsync', () => {
        it('appropriately executes update privacy policy', async () => {
            const companionProvider = createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider();
            const viewerContext = new ViewerContext_1.default(companionProvider);
            const data = {
                id: 'what',
            };
            const testEntity = new SimpleTestDenyUpdateEntity(viewerContext, data);
            const canViewerDelete = await SimpleTestDenyUpdateEntity.canViewerDeleteAsync(testEntity);
            expect(canViewerDelete).toBe(true);
        });
        it('denies when policy denies', async () => {
            const companionProvider = createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider();
            const viewerContext = new ViewerContext_1.default(companionProvider);
            const data = {
                id: 'what',
            };
            const testEntity = new SimpleTestDenyDeleteEntity(viewerContext, data);
            const canViewerDelete = await SimpleTestDenyDeleteEntity.canViewerDeleteAsync(testEntity);
            expect(canViewerDelete).toBe(false);
        });
    });
});
const testEntityConfiguration = new EntityConfiguration_1.default({
    idField: 'id',
    tableName: 'blah',
    schema: {
        id: new EntityFields_1.UUIDField({
            columnName: 'custom_id',
        }),
    },
    databaseAdapterFlavor: 'postgres',
    cacheAdapterFlavor: 'redis',
});
class SimpleTestDenyUpdateEntityPrivacyPolicy extends EntityPrivacyPolicy_1.default {
    constructor() {
        super(...arguments);
        this.readRules = [
            new AlwaysAllowPrivacyPolicyRule_1.default(),
        ];
        this.createRules = [
            new AlwaysAllowPrivacyPolicyRule_1.default(),
        ];
        this.updateRules = [
            new AlwaysDenyPrivacyPolicyRule_1.default(),
        ];
        this.deleteRules = [
            new AlwaysAllowPrivacyPolicyRule_1.default(),
        ];
    }
}
class SimpleTestDenyDeleteEntityPrivacyPolicy extends EntityPrivacyPolicy_1.default {
    constructor() {
        super(...arguments);
        this.readRules = [
            new AlwaysAllowPrivacyPolicyRule_1.default(),
        ];
        this.createRules = [
            new AlwaysAllowPrivacyPolicyRule_1.default(),
        ];
        this.updateRules = [
            new AlwaysAllowPrivacyPolicyRule_1.default(),
        ];
        this.deleteRules = [
            new AlwaysDenyPrivacyPolicyRule_1.default(),
        ];
    }
}
class SimpleTestDenyUpdateEntity extends Entity_1.default {
    static getCompanionDefinition() {
        return simpleTestDenyUpdateEntityCompanion;
    }
}
const simpleTestDenyUpdateEntityCompanion = new EntityCompanionProvider_1.EntityCompanionDefinition({
    entityClass: SimpleTestDenyUpdateEntity,
    entityConfiguration: testEntityConfiguration,
    privacyPolicyClass: SimpleTestDenyUpdateEntityPrivacyPolicy,
});
class SimpleTestDenyDeleteEntity extends Entity_1.default {
    static getCompanionDefinition() {
        return simpleTestDenyDeleteEntityCompanion;
    }
}
const simpleTestDenyDeleteEntityCompanion = new EntityCompanionProvider_1.EntityCompanionDefinition({
    entityClass: SimpleTestDenyDeleteEntity,
    entityConfiguration: testEntityConfiguration,
    privacyPolicyClass: SimpleTestDenyDeleteEntityPrivacyPolicy,
});
//# sourceMappingURL=Entity-test.js.map