"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid_1 = require("uuid");
const EntityFields_1 = require("../EntityFields");
class TestFieldDefinition extends EntityFields_1.EntityFieldDefinition {
    validateInputValueInternal(value) {
        return value === 'helloworld';
    }
}
describe(EntityFields_1.EntityFieldDefinition, () => {
    it('returns correct column name and defaults cache to false', () => {
        const fieldDefinition1 = new TestFieldDefinition({ columnName: 'wat' });
        expect(fieldDefinition1.columnName).toEqual('wat');
        expect(fieldDefinition1.cache).toEqual(false);
        const fieldDefinition2 = new TestFieldDefinition({ columnName: 'wat', cache: true });
        expect(fieldDefinition2.columnName).toEqual('wat');
        expect(fieldDefinition2.cache).toEqual(true);
    });
    test('validator returns true when value is null', () => {
        const fieldDefinition = new TestFieldDefinition({ columnName: 'wat', cache: true });
        expect(fieldDefinition.validateInputValue(null)).toBe(true);
    });
    test('validator returns true when value is undefined', () => {
        const fieldDefinition = new TestFieldDefinition({ columnName: 'wat', cache: true });
        expect(fieldDefinition.validateInputValue(undefined)).toBe(true);
    });
    test('validator returns false when value is invalid', () => {
        const fieldDefinition = new TestFieldDefinition({ columnName: 'wat', cache: true });
        expect(fieldDefinition.validateInputValue('nothelloworld')).toBe(false);
    });
    test('validator returns true when value is valid', () => {
        const fieldDefinition = new TestFieldDefinition({ columnName: 'wat', cache: true });
        expect(fieldDefinition.validateInputValue('helloworld')).toBe(true);
    });
});
const describeFieldTestCase = (fieldDefinition, validValues, invalidValues) => {
    describe(fieldDefinition.constructor.name, () => {
        if (validValues.length > 0) {
            test.each(validValues)(`${fieldDefinition.constructor.name}.valid %p`, (value) => {
                expect(fieldDefinition.validateInputValue(value)).toBe(true);
            });
        }
        if (invalidValues.length > 0) {
            test.each(invalidValues)(`${fieldDefinition.constructor.name}.invalid %p`, (value) => {
                expect(fieldDefinition.validateInputValue(value)).toBe(false);
            });
        }
    });
};
describeFieldTestCase(new EntityFields_1.StringField({ columnName: 'wat' }), ['hello', ''], [1, true, {}, [[]]]);
describeFieldTestCase(new EntityFields_1.UUIDField({ columnName: 'wat' }), [uuid_1.v1(), uuid_1.v3('wat', uuid_1.v3.DNS), uuid_1.v4(), uuid_1.v5('wat', uuid_1.v5.DNS)], [uuid_1.v4().replace('-', ''), '', 'hello']);
describeFieldTestCase(new EntityFields_1.DateField({ columnName: 'wat' }), [new Date()], [Date.now()]);
describeFieldTestCase(new EntityFields_1.BooleanField({ columnName: 'wat' }), [true, false], [0, 1, '']);
describeFieldTestCase(new EntityFields_1.IntField({ columnName: 'wat' }), [1], ['1', 0.5, true]);
describeFieldTestCase(new EntityFields_1.FloatField({ columnName: 'wat' }), [1, 0.5, -0.5], ['1']);
describeFieldTestCase(new EntityFields_1.StringArrayField({ columnName: 'wat' }), [[['what']], [[]]], // jest test cases need extra wrapping array
['hello']);
describeFieldTestCase(new EntityFields_1.JSONObjectField({ columnName: 'wat' }), [{}], [true, 'hello']);
describeFieldTestCase(new EntityFields_1.EnumField({ columnName: 'wat' }), ['hello', 1], [true]);
describeFieldTestCase(new EntityFields_1.JSONArrayField({ columnName: 'wat' }), [[[1, 2]], [['hello']]], // jest test cases need extra wrapping array
[1, 'hello']);
describeFieldTestCase(new EntityFields_1.MaybeJSONArrayField({ columnName: 'wat' }), [1, 'hello', [['hello']]], // jest test cases need extra wrapping array
[]);
//# sourceMappingURL=EntityFields-test.js.map