"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const results_1 = require("@expo/results");
const ts_mockito_1 = require("ts-mockito");
const uuid_1 = require("uuid");
const EntityLoader_1 = __importDefault(require("../EntityLoader"));
const ViewerContext_1 = __importDefault(require("../ViewerContext"));
const entityUtils_1 = require("../entityUtils");
const EntityDataManager_1 = __importDefault(require("../internal/EntityDataManager"));
const ReadThroughEntityCache_1 = __importDefault(require("../internal/ReadThroughEntityCache"));
const TestEntity_1 = __importStar(require("../testfixtures/TestEntity"));
const StubCacheAdapter_1 = require("../utils/testing/StubCacheAdapter");
const StubDatabaseAdapter_1 = __importDefault(require("../utils/testing/StubDatabaseAdapter"));
const StubQueryContextProvider_1 = __importDefault(require("../utils/testing/StubQueryContextProvider"));
describe(EntityLoader_1.default, () => {
    it('loads entities', async () => {
        var _a;
        const dateToInsert = new Date();
        const viewerContext = ts_mockito_1.instance(ts_mockito_1.mock(ViewerContext_1.default));
        const queryContext = StubQueryContextProvider_1.default.getQueryContext();
        const id1 = uuid_1.v4();
        const id2 = uuid_1.v4();
        const databaseAdapter = new StubDatabaseAdapter_1.default(TestEntity_1.testEntityConfiguration, StubDatabaseAdapter_1.default.convertFieldObjectsToDataStore(TestEntity_1.testEntityConfiguration, new Map([
            [
                TestEntity_1.testEntityConfiguration.tableName,
                [
                    {
                        customIdField: id1,
                        testIndexedField: 'h1',
                        intField: 5,
                        stringField: 'huh',
                        dateField: dateToInsert,
                        nullableField: null,
                    },
                    {
                        customIdField: id2,
                        testIndexedField: 'h2',
                        intField: 3,
                        stringField: 'huh',
                        dateField: dateToInsert,
                        nullableField: null,
                    },
                ],
            ],
        ])));
        const privacyPolicy = new TestEntity_1.TestEntityPrivacyPolicy();
        const cacheAdapterProvider = new StubCacheAdapter_1.NoCacheStubCacheAdapterProvider();
        const cacheAdapter = cacheAdapterProvider.getCacheAdapter(TestEntity_1.testEntityConfiguration);
        const entityCache = new ReadThroughEntityCache_1.default(TestEntity_1.testEntityConfiguration, cacheAdapter);
        const dataManager = new EntityDataManager_1.default(databaseAdapter, entityCache, StubQueryContextProvider_1.default, ts_mockito_1.instance(ts_mockito_1.mock()), TestEntity_1.default.name);
        const entityLoader = new EntityLoader_1.default(viewerContext, queryContext, TestEntity_1.testEntityConfiguration, TestEntity_1.default, privacyPolicy, dataManager);
        const entity = await results_1.enforceAsyncResult(entityLoader.loadByIDAsync(id1));
        expect(entity.getID()).toEqual(id1);
        expect(entity.getField('dateField')).toEqual(dateToInsert);
        const entities = await entityUtils_1.enforceResultsAsync(entityLoader.loadManyByFieldEqualingAsync('stringField', 'huh'));
        expect(entities.map((m) => m.getID())).toEqual([id1, id2]);
        const entityResultNumber3 = await entityLoader.loadByFieldEqualingAsync('intField', 3);
        expect(entityResultNumber3).not.toBeNull();
        expect(entityResultNumber3.enforceValue().getID()).toEqual(id2);
        const entityResultNumber4 = await entityLoader.loadByFieldEqualingAsync('intField', 4);
        expect(entityResultNumber4).toBeNull();
        const entityResultDuplicateValues = await entityLoader
            .enforcing()
            .loadManyByFieldEqualingManyAsync('stringField', ['huh', 'huh']);
        expect(entityResultDuplicateValues.size).toBe(1);
        expect((_a = entityResultDuplicateValues.get('huh')) === null || _a === void 0 ? void 0 : _a.map((m) => m.getID())).toEqual([id1, id2]);
        await expect(entityLoader.loadByFieldEqualingAsync('stringField', 'huh')).rejects.toThrowError('loadByFieldEqualing: Multiple entities of type TestEntity found for stringField=huh');
        await expect(entityLoader.loadByIDNullableAsync(uuid_1.v4())).resolves.toBeNull();
        await expect(entityLoader.loadByIDNullableAsync(id1)).resolves.not.toBeNull();
        await expect(entityLoader.loadByIDAsync('not-a-uuid')).rejects.toThrowError('Entity field not valid: TestEntity (customIdField = not-a-uuid)');
    });
    it('loads entities with loadManyByFieldEqualityConjunction', async () => {
        const privacyPolicy = new TestEntity_1.TestEntityPrivacyPolicy();
        const spiedPrivacyPolicy = ts_mockito_1.spy(privacyPolicy);
        const viewerContext = ts_mockito_1.instance(ts_mockito_1.mock(ViewerContext_1.default));
        const queryContext = StubQueryContextProvider_1.default.getQueryContext();
        const id1 = uuid_1.v4();
        const id2 = uuid_1.v4();
        const id3 = uuid_1.v4();
        const databaseAdapter = new StubDatabaseAdapter_1.default(TestEntity_1.testEntityConfiguration, StubDatabaseAdapter_1.default.convertFieldObjectsToDataStore(TestEntity_1.testEntityConfiguration, new Map([
            [
                TestEntity_1.testEntityConfiguration.tableName,
                [
                    {
                        customIdField: id1,
                        stringField: 'huh',
                        intField: 4,
                        testIndexedField: '4',
                        dateField: new Date(),
                        nullableField: null,
                    },
                    {
                        customIdField: id2,
                        stringField: 'huh',
                        intField: 4,
                        testIndexedField: '5',
                        dateField: new Date(),
                        nullableField: null,
                    },
                    {
                        customIdField: id3,
                        stringField: 'huh2',
                        intField: 4,
                        testIndexedField: '6',
                        dateField: new Date(),
                        nullableField: null,
                    },
                ],
            ],
        ])));
        const cacheAdapterProvider = new StubCacheAdapter_1.NoCacheStubCacheAdapterProvider();
        const cacheAdapter = cacheAdapterProvider.getCacheAdapter(TestEntity_1.testEntityConfiguration);
        const entityCache = new ReadThroughEntityCache_1.default(TestEntity_1.testEntityConfiguration, cacheAdapter);
        const dataManager = new EntityDataManager_1.default(databaseAdapter, entityCache, StubQueryContextProvider_1.default, ts_mockito_1.instance(ts_mockito_1.mock()), TestEntity_1.default.name);
        const entityLoader = new EntityLoader_1.default(viewerContext, queryContext, TestEntity_1.testEntityConfiguration, TestEntity_1.default, privacyPolicy, dataManager);
        const entities = await entityUtils_1.enforceResultsAsync(entityLoader.loadManyByFieldEqualityConjunctionAsync([
            {
                fieldName: 'stringField',
                fieldValue: 'huh',
            },
            {
                fieldName: 'intField',
                fieldValue: 4,
            },
        ]));
        expect(entities).toHaveLength(2);
        ts_mockito_1.verify(spiedPrivacyPolicy.authorizeReadAsync(viewerContext, queryContext, ts_mockito_1.anyOfClass(TestEntity_1.default))).twice();
        await expect(entityLoader.loadManyByFieldEqualityConjunctionAsync([
            { fieldName: 'customIdField', fieldValue: 'not-a-uuid' },
        ])).rejects.toThrowError('Entity field not valid: TestEntity (customIdField = not-a-uuid)');
    });
    it('authorizes loaded entities', async () => {
        const privacyPolicy = new TestEntity_1.TestEntityPrivacyPolicy();
        const spiedPrivacyPolicy = ts_mockito_1.spy(privacyPolicy);
        const viewerContext = ts_mockito_1.instance(ts_mockito_1.mock(ViewerContext_1.default));
        const queryContext = StubQueryContextProvider_1.default.getQueryContext();
        const id1 = uuid_1.v4();
        const databaseAdapter = new StubDatabaseAdapter_1.default(TestEntity_1.testEntityConfiguration, StubDatabaseAdapter_1.default.convertFieldObjectsToDataStore(TestEntity_1.testEntityConfiguration, new Map([
            [
                TestEntity_1.testEntityConfiguration.tableName,
                [
                    {
                        customIdField: id1,
                        stringField: 'huh',
                        testIndexedField: '1',
                        intField: 3,
                        dateField: new Date(),
                        nullableField: null,
                    },
                ],
            ],
        ])));
        const cacheAdapterProvider = new StubCacheAdapter_1.NoCacheStubCacheAdapterProvider();
        const cacheAdapter = cacheAdapterProvider.getCacheAdapter(TestEntity_1.testEntityConfiguration);
        const entityCache = new ReadThroughEntityCache_1.default(TestEntity_1.testEntityConfiguration, cacheAdapter);
        const dataManager = new EntityDataManager_1.default(databaseAdapter, entityCache, StubQueryContextProvider_1.default, ts_mockito_1.instance(ts_mockito_1.mock()), TestEntity_1.default.name);
        const entityLoader = new EntityLoader_1.default(viewerContext, queryContext, TestEntity_1.testEntityConfiguration, TestEntity_1.default, privacyPolicy, dataManager);
        const entity = await results_1.enforceAsyncResult(entityLoader.loadByIDAsync(id1));
        ts_mockito_1.verify(spiedPrivacyPolicy.authorizeReadAsync(viewerContext, queryContext, entity)).once();
    });
    it('invalidates upon invalidate one', async () => {
        const viewerContext = ts_mockito_1.instance(ts_mockito_1.mock(ViewerContext_1.default));
        const queryContext = StubQueryContextProvider_1.default.getQueryContext();
        const privacyPolicy = ts_mockito_1.instance(ts_mockito_1.mock(TestEntity_1.TestEntityPrivacyPolicy));
        const dataManagerMock = ts_mockito_1.mock();
        const dataManagerInstance = ts_mockito_1.instance(dataManagerMock);
        const id1 = uuid_1.v4();
        const entityLoader = new EntityLoader_1.default(viewerContext, queryContext, TestEntity_1.testEntityConfiguration, TestEntity_1.default, privacyPolicy, dataManagerInstance);
        await entityLoader.invalidateFieldsAsync({ customIdField: id1 });
        ts_mockito_1.verify(dataManagerMock.invalidateObjectFieldsAsync(ts_mockito_1.deepEqual({ customIdField: id1 }))).once();
    });
    it('invalidates upon invalidate by field', async () => {
        const viewerContext = ts_mockito_1.instance(ts_mockito_1.mock(ViewerContext_1.default));
        const queryContext = StubQueryContextProvider_1.default.getQueryContext();
        const privacyPolicy = ts_mockito_1.instance(ts_mockito_1.mock(TestEntity_1.TestEntityPrivacyPolicy));
        const dataManagerMock = ts_mockito_1.mock();
        const dataManagerInstance = ts_mockito_1.instance(dataManagerMock);
        const id1 = uuid_1.v4();
        const entityLoader = new EntityLoader_1.default(viewerContext, queryContext, TestEntity_1.testEntityConfiguration, TestEntity_1.default, privacyPolicy, dataManagerInstance);
        await entityLoader.invalidateFieldsAsync({ customIdField: id1 });
        ts_mockito_1.verify(dataManagerMock.invalidateObjectFieldsAsync(ts_mockito_1.deepEqual({ customIdField: id1 }))).once();
    });
    it('invalidates upon invalidate by entity', async () => {
        const viewerContext = ts_mockito_1.instance(ts_mockito_1.mock(ViewerContext_1.default));
        const queryContext = StubQueryContextProvider_1.default.getQueryContext();
        const privacyPolicy = ts_mockito_1.instance(ts_mockito_1.mock(TestEntity_1.TestEntityPrivacyPolicy));
        const dataManagerMock = ts_mockito_1.mock();
        const dataManagerInstance = ts_mockito_1.instance(dataManagerMock);
        const id1 = uuid_1.v4();
        const entityMock = ts_mockito_1.mock(TestEntity_1.default);
        ts_mockito_1.when(entityMock.getAllDatabaseFields()).thenReturn({ customIdField: id1 });
        const entityInstance = ts_mockito_1.instance(entityMock);
        const entityLoader = new EntityLoader_1.default(viewerContext, queryContext, TestEntity_1.testEntityConfiguration, TestEntity_1.default, privacyPolicy, dataManagerInstance);
        await entityLoader.invalidateEntityAsync(entityInstance);
        ts_mockito_1.verify(dataManagerMock.invalidateObjectFieldsAsync(ts_mockito_1.deepEqual({ customIdField: id1 }))).once();
    });
    it('returns error result when not allowed', async () => {
        const viewerContext = ts_mockito_1.instance(ts_mockito_1.mock(ViewerContext_1.default));
        const queryContext = StubQueryContextProvider_1.default.getQueryContext();
        const privacyPolicyMock = ts_mockito_1.mock(TestEntity_1.TestEntityPrivacyPolicy);
        const dataManagerMock = ts_mockito_1.mock();
        const id1 = uuid_1.v4();
        ts_mockito_1.when(dataManagerMock.loadManyByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything(), ts_mockito_1.anything())).thenResolve(new Map().set(id1, [{ customIdField: id1 }]));
        const rejectionError = new Error();
        ts_mockito_1.when(privacyPolicyMock.authorizeReadAsync(viewerContext, queryContext, ts_mockito_1.anyOfClass(TestEntity_1.default))).thenReject(rejectionError);
        const privacyPolicy = ts_mockito_1.instance(privacyPolicyMock);
        const dataManagerInstance = ts_mockito_1.instance(dataManagerMock);
        const entityLoader = new EntityLoader_1.default(viewerContext, queryContext, TestEntity_1.testEntityConfiguration, TestEntity_1.default, privacyPolicy, dataManagerInstance);
        const entityResult = await entityLoader.loadByIDAsync(id1);
        expect(entityResult.ok).toBe(false);
        expect(entityResult.reason).toEqual(rejectionError);
        expect(entityResult.value).toBe(undefined);
    });
    it('throws upon database adapter error', async () => {
        const viewerContext = ts_mockito_1.instance(ts_mockito_1.mock(ViewerContext_1.default));
        const queryContext = StubQueryContextProvider_1.default.getQueryContext();
        const privacyPolicy = ts_mockito_1.instance(ts_mockito_1.mock(TestEntity_1.TestEntityPrivacyPolicy));
        const dataManagerMock = ts_mockito_1.mock();
        const error = new Error();
        ts_mockito_1.when(dataManagerMock.loadManyByFieldEqualingAsync(ts_mockito_1.anything(), ts_mockito_1.anything(), ts_mockito_1.anything())).thenReject(error);
        const dataManagerInstance = ts_mockito_1.instance(dataManagerMock);
        const entityLoader = new EntityLoader_1.default(viewerContext, queryContext, TestEntity_1.testEntityConfiguration, TestEntity_1.default, privacyPolicy, dataManagerInstance);
        const loadByValue = uuid_1.v4();
        await expect(entityLoader.loadByIDAsync(loadByValue)).rejects.toEqual(error);
        await expect(entityLoader.enforcing().loadByIDAsync(loadByValue)).rejects.toEqual(error);
        await expect(entityLoader.loadManyByIDsAsync([loadByValue])).rejects.toEqual(error);
        await expect(entityLoader.loadManyByIDsAsync([loadByValue])).rejects.toEqual(error);
        await expect(entityLoader.loadManyByFieldEqualingAsync('customIdField', loadByValue)).rejects.toEqual(error);
        await expect(entityLoader.enforcing().loadManyByFieldEqualingAsync('customIdField', loadByValue)).rejects.toEqual(error);
        await expect(entityLoader.loadManyByFieldEqualingManyAsync('customIdField', [loadByValue])).rejects.toEqual(error);
        await expect(entityLoader.enforcing().loadManyByFieldEqualingManyAsync('customIdField', [loadByValue])).rejects.toEqual(error);
    });
});
//# sourceMappingURL=EntityLoader-test.js.map