"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_mockito_1 = require("ts-mockito");
const EntityQueryContext_1 = require("../EntityQueryContext");
const EntitySecondaryCacheLoader_1 = __importDefault(require("../EntitySecondaryCacheLoader"));
const ViewerContext_1 = __importDefault(require("../ViewerContext"));
const SimpleTestEntity_1 = __importDefault(require("../testfixtures/SimpleTestEntity"));
const createUnitTestEntityCompanionProvider_1 = require("../utils/testing/createUnitTestEntityCompanionProvider");
class TestSecondaryRedisCacheLoader extends EntitySecondaryCacheLoader_1.default {
    async fetchObjectsFromDatabaseAsync(_loadParamsArray) {
        // unused
        return new Map();
    }
}
describe(EntitySecondaryCacheLoader_1.default, () => {
    describe('loadManyAsync', () => {
        it('calls into secondary cache with correct params', async () => {
            const vc1 = new ViewerContext_1.default(createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider());
            const createdEntity = await SimpleTestEntity_1.default.creator(vc1).enforceCreateAsync();
            const loadParams = { id: createdEntity.getID() };
            const secondaryEntityCacheMock = ts_mockito_1.mock();
            ts_mockito_1.when(secondaryEntityCacheMock.loadManyThroughAsync(ts_mockito_1.deepEqual([loadParams]), ts_mockito_1.anything())).thenResolve(new Map());
            const secondaryEntityCache = ts_mockito_1.instance(secondaryEntityCacheMock);
            const secondaryCacheLoader = new TestSecondaryRedisCacheLoader(secondaryEntityCache, SimpleTestEntity_1.default.loader(vc1));
            await secondaryCacheLoader.loadManyAsync([loadParams]);
            ts_mockito_1.verify(secondaryEntityCacheMock.loadManyThroughAsync(ts_mockito_1.deepEqual([loadParams]), ts_mockito_1.anything())).once();
        });
        it('constructs and authorizes entities', async () => {
            var _a;
            const vc1 = new ViewerContext_1.default(createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider());
            const createdEntity = await SimpleTestEntity_1.default.creator(vc1).enforceCreateAsync();
            const loadParams = { id: createdEntity.getID() };
            const secondaryEntityCacheMock = ts_mockito_1.mock();
            ts_mockito_1.when(secondaryEntityCacheMock.loadManyThroughAsync(ts_mockito_1.deepEqual([loadParams]), ts_mockito_1.anything())).thenResolve(new Map([[loadParams, createdEntity.getAllFields()]]));
            const secondaryEntityCache = ts_mockito_1.instance(secondaryEntityCacheMock);
            const loader = SimpleTestEntity_1.default.loader(vc1);
            const spiedPrivacyPolicy = ts_mockito_1.spy(loader['privacyPolicy']);
            const secondaryCacheLoader = new TestSecondaryRedisCacheLoader(secondaryEntityCache, loader);
            const result = await secondaryCacheLoader.loadManyAsync([loadParams]);
            expect((_a = result.get(loadParams)) === null || _a === void 0 ? void 0 : _a.enforceValue().getID()).toEqual(createdEntity.getID());
            ts_mockito_1.verify(spiedPrivacyPolicy.authorizeReadAsync(vc1, ts_mockito_1.anyOfClass(EntityQueryContext_1.EntityNonTransactionalQueryContext), ts_mockito_1.anything())).once();
        });
    });
    describe('invalidateManyAsync', () => {
        it('calls invalidate on the secondary cache', async () => {
            const vc1 = new ViewerContext_1.default(createUnitTestEntityCompanionProvider_1.createUnitTestEntityCompanionProvider());
            const createdEntity = await SimpleTestEntity_1.default.creator(vc1).enforceCreateAsync();
            const loadParams = { id: createdEntity.getID() };
            const secondaryEntityCacheMock = ts_mockito_1.mock();
            const secondaryEntityCache = ts_mockito_1.instance(secondaryEntityCacheMock);
            const loader = SimpleTestEntity_1.default.loader(vc1);
            const secondaryCacheLoader = new TestSecondaryRedisCacheLoader(secondaryEntityCache, loader);
            await secondaryCacheLoader.invalidateManyAsync([loadParams]);
            ts_mockito_1.verify(secondaryEntityCacheMock.invalidateManyAsync(ts_mockito_1.deepEqual([loadParams]))).once();
        });
    });
});
//# sourceMappingURL=EntitySecondaryCacheLoader-test.js.map