import EntityConfiguration from '../EntityConfiguration';
export interface FieldTransformer<T> {
    /**
     * Transformation to apply when a value is read from an adapter.
     */
    read?: (value: any) => T | null;
    /**
     * Transformation to apply when a value is written to an adapter.
     */
    write?: (value: T | null) => any;
}
/**
 * Map from concrete EntityFieldDefinition implementation class name to field transformer.
 */
export declare type FieldTransformerMap = Map<string, FieldTransformer<any>>;
export declare const getDatabaseFieldForEntityField: <TFields>(entityConfiguration: EntityConfiguration<TFields>, entityField: keyof TFields) => string;
export declare const transformDatabaseObjectToFields: <TFields>(entityConfiguration: EntityConfiguration<TFields>, fieldTransformerMap: FieldTransformerMap, databaseObject: {
    [key: string]: any;
}) => Readonly<TFields>;
export declare const transformFieldsToDatabaseObject: <TFields>(entityConfiguration: EntityConfiguration<TFields>, fieldTransformerMap: FieldTransformerMap, fields: Readonly<Partial<TFields>>) => object;
export declare const transformCacheObjectToFields: <TFields>(entityConfiguration: EntityConfiguration<TFields>, fieldTransformerMap: FieldTransformerMap, cacheObject: {
    [key: string]: any;
}) => Readonly<TFields>;
export declare const transformFieldsToCacheObject: <TFields>(entityConfiguration: EntityConfiguration<TFields>, fieldTransformerMap: FieldTransformerMap, fields: Readonly<Partial<TFields>>) => object;
