"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_mockito_1 = require("ts-mockito");
const EntityConfiguration_1 = __importDefault(require("../../EntityConfiguration"));
const EntityFields_1 = require("../../EntityFields");
const ReadThroughEntityCache_1 = __importStar(require("../ReadThroughEntityCache"));
const makeEntityConfiguration = (cacheIdField) => new EntityConfiguration_1.default({
    idField: 'id',
    tableName: 'blah',
    schema: {
        id: new EntityFields_1.UUIDField({ columnName: 'id', cache: cacheIdField }),
    },
    databaseAdapterFlavor: 'postgres',
    cacheAdapterFlavor: 'redis',
});
const createIdFetcher = (ids) => async (fetcherFieldValues) => {
    const results = new Map();
    fetcherFieldValues.forEach((v) => {
        if (ids.includes(v)) {
            results.set(v, [{ id: v }]);
        }
        else {
            results.set(v, []);
        }
    });
    return results;
};
describe(ReadThroughEntityCache_1.default, () => {
    describe('readManyThroughAsync', () => {
        it('fetches from DB upon cache miss and caches the result', async () => {
            const cacheAdapterMock = ts_mockito_1.mock();
            ts_mockito_1.when(cacheAdapterMock.getFieldTransformerMap()).thenReturn(new Map());
            const cacheAdapter = ts_mockito_1.instance(cacheAdapterMock);
            const entityCache = new ReadThroughEntityCache_1.default(makeEntityConfiguration(true), cacheAdapter);
            const fetcher = createIdFetcher(['wat', 'who']);
            ts_mockito_1.when(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['wat', 'who']))).thenResolve(new Map([
                ['wat', { status: ReadThroughEntityCache_1.CacheStatus.MISS }],
                ['who', { status: ReadThroughEntityCache_1.CacheStatus.MISS }],
            ]));
            const result = await entityCache.readManyThroughAsync('id', ['wat', 'who'], fetcher);
            ts_mockito_1.verify(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['wat', 'who']))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheManyAsync('id', ts_mockito_1.deepEqual(new Map([
                ['wat', { id: 'wat' }],
                ['who', { id: 'who' }],
            ])))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheDBMissesAsync('id', ts_mockito_1.deepEqual([]))).once();
            expect(result).toEqual(new Map([
                ['wat', [{ id: 'wat' }]],
                ['who', [{ id: 'who' }]],
            ]));
        });
        it('does not fetch from the DB or cache results when all cache fetches are hits', async () => {
            const cacheAdapterMock = ts_mockito_1.mock();
            ts_mockito_1.when(cacheAdapterMock.getFieldTransformerMap()).thenReturn(new Map());
            const cacheAdapter = ts_mockito_1.instance(cacheAdapterMock);
            const entityCache = new ReadThroughEntityCache_1.default(makeEntityConfiguration(true), cacheAdapter);
            const fetcher = createIdFetcher(['wat', 'who']);
            ts_mockito_1.when(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['wat', 'who']))).thenResolve(new Map([
                ['wat', { status: ReadThroughEntityCache_1.CacheStatus.HIT, item: { id: 'wat' } }],
                ['who', { status: ReadThroughEntityCache_1.CacheStatus.HIT, item: { id: 'who' } }],
            ]));
            const result = await entityCache.readManyThroughAsync('id', ['wat', 'who'], fetcher);
            ts_mockito_1.verify(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['wat', 'who']))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheManyAsync('id', ts_mockito_1.deepEqual(new Map([
                ['wat', { id: 'wat' }],
                ['who', { id: 'who' }],
            ])))).never();
            ts_mockito_1.verify(cacheAdapterMock.cacheDBMissesAsync('id', ts_mockito_1.deepEqual([]))).never();
            expect(result).toEqual(new Map([
                ['wat', [{ id: 'wat' }]],
                ['who', [{ id: 'who' }]],
            ]));
        });
        it('negatively caches db misses', async () => {
            const cacheAdapterMock = ts_mockito_1.mock();
            ts_mockito_1.when(cacheAdapterMock.getFieldTransformerMap()).thenReturn(new Map());
            const cacheAdapter = ts_mockito_1.instance(cacheAdapterMock);
            const entityCache = new ReadThroughEntityCache_1.default(makeEntityConfiguration(true), cacheAdapter);
            // simulate db miss
            const fetcher = createIdFetcher(['wat', 'who']);
            ts_mockito_1.when(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['why']))).thenResolve(new Map([['why', { status: ReadThroughEntityCache_1.CacheStatus.MISS }]]));
            const result = await entityCache.readManyThroughAsync('id', ['why'], fetcher);
            ts_mockito_1.verify(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['why']))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheManyAsync('id', ts_mockito_1.deepEqual(new Map()))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheDBMissesAsync('id', ts_mockito_1.deepEqual(['why']))).once();
            expect(result).toEqual(new Map());
        });
        it('does not return or fetch negatively cached results from DB', async () => {
            const cacheAdapterMock = ts_mockito_1.mock();
            ts_mockito_1.when(cacheAdapterMock.getFieldTransformerMap()).thenReturn(new Map());
            const cacheAdapter = ts_mockito_1.instance(cacheAdapterMock);
            const entityCache = new ReadThroughEntityCache_1.default(makeEntityConfiguration(true), cacheAdapter);
            const fetcher = createIdFetcher([]);
            ts_mockito_1.when(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['why']))).thenResolve(new Map([['why', { status: ReadThroughEntityCache_1.CacheStatus.NEGATIVE }]]));
            const result = await entityCache.readManyThroughAsync('id', ['why'], fetcher);
            ts_mockito_1.verify(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['why']))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheManyAsync('id', ts_mockito_1.anything())).never();
            ts_mockito_1.verify(cacheAdapterMock.cacheDBMissesAsync('id', ts_mockito_1.anything())).never();
            expect(result).toEqual(new Map());
        });
        it('does a mix and match of hit, miss, and negative', async () => {
            const cacheAdapterMock = ts_mockito_1.mock();
            ts_mockito_1.when(cacheAdapterMock.getFieldTransformerMap()).thenReturn(new Map());
            const cacheAdapter = ts_mockito_1.instance(cacheAdapterMock);
            const entityCache = new ReadThroughEntityCache_1.default(makeEntityConfiguration(true), cacheAdapter);
            const fetcher = createIdFetcher(['wat', 'who', 'why']);
            ts_mockito_1.when(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['wat', 'who', 'why', 'how']))).thenResolve(new Map([
                ['wat', { status: ReadThroughEntityCache_1.CacheStatus.MISS }],
                ['who', { status: ReadThroughEntityCache_1.CacheStatus.NEGATIVE }],
                ['why', { status: ReadThroughEntityCache_1.CacheStatus.HIT, item: { id: 'why' } }],
                ['how', { status: ReadThroughEntityCache_1.CacheStatus.MISS }],
            ]));
            const result = await entityCache.readManyThroughAsync('id', ['wat', 'who', 'why', 'how'], fetcher);
            ts_mockito_1.verify(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['wat', 'who', 'why', 'how']))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheManyAsync('id', ts_mockito_1.deepEqual(new Map([['wat', { id: 'wat' }]])))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheDBMissesAsync('id', ts_mockito_1.deepEqual(['how']))).once();
            expect(result).toEqual(new Map([
                ['wat', [{ id: 'wat' }]],
                ['why', [{ id: 'why' }]],
            ]));
        });
        it('does not call into cache for field that is not cacheable', async () => {
            const cacheAdapterMock = ts_mockito_1.mock();
            ts_mockito_1.when(cacheAdapterMock.getFieldTransformerMap()).thenReturn(new Map());
            const cacheAdapter = ts_mockito_1.instance(cacheAdapterMock);
            const entityCache = new ReadThroughEntityCache_1.default(makeEntityConfiguration(false), cacheAdapter);
            const fetcher = createIdFetcher(['wat']);
            const result = await entityCache.readManyThroughAsync('id', ['wat'], fetcher);
            ts_mockito_1.verify(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.anything())).never();
            expect(result).toEqual(new Map([['wat', [{ id: 'wat' }]]]));
        });
        it('transforms fields for cache storage', async () => {
            const cacheAdapterMock = ts_mockito_1.mock();
            ts_mockito_1.when(cacheAdapterMock.getFieldTransformerMap()).thenReturn(new Map([
                [
                    EntityFields_1.UUIDField.name,
                    {
                        read: (val) => val.split('-')[0],
                        write: (val) => `${val}-in-cache`,
                    },
                ],
            ]));
            const cacheAdapter = ts_mockito_1.instance(cacheAdapterMock);
            const entityCache = new ReadThroughEntityCache_1.default(makeEntityConfiguration(true), cacheAdapter);
            const fetcher = createIdFetcher(['wat', 'who']);
            ts_mockito_1.when(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['wat', 'who']))).thenResolve(new Map([
                ['wat', { status: ReadThroughEntityCache_1.CacheStatus.MISS }],
                ['who', { status: ReadThroughEntityCache_1.CacheStatus.HIT, item: { id: 'who-in-cache' } }],
            ]));
            const result = await entityCache.readManyThroughAsync('id', ['wat', 'who'], fetcher);
            ts_mockito_1.verify(cacheAdapterMock.loadManyAsync('id', ts_mockito_1.deepEqual(['wat', 'who']))).once();
            ts_mockito_1.verify(cacheAdapterMock.cacheManyAsync('id', ts_mockito_1.deepEqual(new Map([['wat', { id: 'wat-in-cache' }]])))).once();
            expect(result).toEqual(new Map([
                ['wat', [{ id: 'wat' }]],
                ['who', [{ id: 'who' }]],
            ]));
        });
    });
    describe('invalidateManyAsync', () => {
        it('calls cache adapter invalidate', async () => {
            const cacheAdapterMock = ts_mockito_1.mock();
            const cacheAdapter = ts_mockito_1.instance(cacheAdapterMock);
            const entityCache = new ReadThroughEntityCache_1.default(makeEntityConfiguration(true), cacheAdapter);
            await entityCache.invalidateManyAsync('id', ['wat']);
            ts_mockito_1.verify(cacheAdapterMock.invalidateManyAsync('id', ts_mockito_1.deepEqual(['wat']))).once();
        });
    });
});
//# sourceMappingURL=ReadThroughEntityCache-test.js.map