import { EntityQueryContext } from '../../EntityQueryContext';
import ReadonlyEntity from '../../ReadonlyEntity';
import ViewerContext from '../../ViewerContext';
import PrivacyPolicyRule from '../../rules/PrivacyPolicyRule';
export interface Case<TFields, TID extends NonNullable<TFields[TSelectedFields]>, TViewerContext extends ViewerContext, TEntity extends ReadonlyEntity<TFields, TID, TViewerContext, TSelectedFields>, TSelectedFields extends keyof TFields> {
    viewerContext: TViewerContext;
    queryContext: EntityQueryContext;
    entity: TEntity;
}
declare type CaseMap<TFields, TID extends NonNullable<TFields[TSelectedFields]>, TViewerContext extends ViewerContext, TEntity extends ReadonlyEntity<TFields, TID, TViewerContext, TSelectedFields>, TSelectedFields extends keyof TFields> = Map<string, () => Promise<Case<TFields, TID, TViewerContext, TEntity, TSelectedFields>>>;
/**
 * Useful for defining test cases that have async preconditions.
 */
export declare const describePrivacyPolicyRuleWithAsyncTestCase: <TFields, TID extends NonNullable<TFields[TSelectedFields]>, TViewerContext extends ViewerContext, TEntity extends ReadonlyEntity<TFields, TID, TViewerContext, TSelectedFields>, TSelectedFields extends keyof TFields = keyof TFields>(privacyPolicyRule: PrivacyPolicyRule<TFields, TID, TViewerContext, TEntity, TSelectedFields>, { allowCases, skipCases, denyCases, }: {
    allowCases?: CaseMap<TFields, TID, TViewerContext, TEntity, TSelectedFields> | undefined;
    skipCases?: CaseMap<TFields, TID, TViewerContext, TEntity, TSelectedFields> | undefined;
    denyCases?: CaseMap<TFields, TID, TViewerContext, TEntity, TSelectedFields> | undefined;
}) => void;
/**
 * For test simple privacy rules that don't have complex async preconditions.
 */
export declare const describePrivacyPolicyRule: <TFields, TID extends NonNullable<TFields[TSelectedFields]>, TViewerContext extends ViewerContext, TEntity extends ReadonlyEntity<TFields, TID, TViewerContext, TSelectedFields>, TSelectedFields extends keyof TFields = keyof TFields>(privacyPolicyRule: PrivacyPolicyRule<TFields, TID, TViewerContext, TEntity, TSelectedFields>, { allowCases, skipCases, denyCases, }: {
    allowCases?: Case<TFields, TID, TViewerContext, TEntity, TSelectedFields>[] | undefined;
    skipCases?: Case<TFields, TID, TViewerContext, TEntity, TSelectedFields>[] | undefined;
    denyCases?: Case<TFields, TID, TViewerContext, TEntity, TSelectedFields>[] | undefined;
}) => void;
export {};
