"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createUnitTestEntityCompanionProvider = void 0;
const EntityCompanionProvider_1 = __importDefault(require("../../EntityCompanionProvider"));
const NoOpEntityMetricsAdapter_1 = __importDefault(require("../../metrics/NoOpEntityMetricsAdapter"));
const StubCacheAdapter_1 = require("./StubCacheAdapter");
const StubDatabaseAdapterProvider_1 = __importDefault(require("./StubDatabaseAdapterProvider"));
const StubQueryContextProvider_1 = __importDefault(require("./StubQueryContextProvider"));
/**
 * Entity companion provider for use in unit tests. All database and cache implementations
 * are replaced with in-memory simulations.
 */
const createUnitTestEntityCompanionProvider = (metricsAdapter = new NoOpEntityMetricsAdapter_1.default()) => {
    return new EntityCompanionProvider_1.default(metricsAdapter, new Map([
        [
            'postgres',
            {
                adapterProvider: new StubDatabaseAdapterProvider_1.default(),
                queryContextProvider: StubQueryContextProvider_1.default,
            },
        ],
    ]), new Map([
        [
            'redis',
            {
                cacheAdapterProvider: new StubCacheAdapter_1.InMemoryFullCacheStubCacheAdapterProvider(),
            },
        ],
    ]));
};
exports.createUnitTestEntityCompanionProvider = createUnitTestEntityCompanionProvider;
//# sourceMappingURL=createUnitTestEntityCompanionProvider.js.map